@@ -1,3 +1,4 @@
+import logging
 import os
 import time
 import traceback
@@ -5,6 +6,7 @@ import uuid
 import warnings
 from typing import Dict, Iterable, List, Mapping, Optional, Union
 
+import groq
 import httpx
 import openai
 import openai.types as openai_types
@@ -26,6 +28,7 @@ from maitai._types import (
 from maitai._utils import (
     convert_open_ai_chat_completion_chunk,
     convert_openai_chat_completion,
+    convert_params_to_groq,
     get_chat_completion_params,
     required_args,
     type_to_response_format_param,
@@ -47,6 +50,8 @@ from maitai.models.key import Key
 from maitai.models.metric import RequestTimingMetric
 from maitai.tools import Tools
 
+logger = logging.getLogger(__name__)
+
 DEFAULT_MAX_RETRIES = 2
 
 
@@ -62,6 +67,7 @@ class Maitai:
         cerebras_api_key: Optional[str] = None,
         azure_api_key: Optional[str] = None,
         sambanova_api_key: Optional[str] = None,
+        gemini_api_key: Optional[str] = None,
         azure_deployment: Optional[str] = None,
         azure_api_version: Optional[str] = None,
         azure_endpoint: Optional[str] = None,
@@ -85,7 +91,7 @@ class Maitai:
             )
         if api_key:
             raise ValueError(
-                "api_key is deprecated. Use provider specific keys instead. Supported variables: openai_api_key, groq_api_key, anthropic_api_key, cerebras_api_key, azure_api_key, sambanova_api_key"
+                "api_key is deprecated. Use provider specific keys instead. Supported variables: openai_api_key, groq_api_key, anthropic_api_key, cerebras_api_key, azure_api_key, sambanova_api_key, gemini_api_key"
             )
         if openai_api_key:
             if not config.auth_keys.openai_api_key:
@@ -129,7 +135,15 @@ class Maitai:
                 )
             else:
                 config.auth_keys.sambanova_api_key.key_value = sambanova_api_key
-        self._client = None
+        if gemini_api_key:
+            if not config.auth_keys.gemini_api_key:
+                config.auth_keys.gemini_api_key = Key(
+                    key_value=gemini_api_key,
+                )
+            else:
+                config.auth_keys.gemini_api_key.key_value = gemini_api_key
+        self._openai_client = None
+        self._groq_client = None
         azure_params = None
         if azure_api_key:
             azure_params = AzureParams(
@@ -137,7 +151,7 @@ class Maitai:
                 deployment=azure_deployment,
                 api_version=azure_api_version,
             )
-            self._client = openai.AzureOpenAI(
+            self._openai_client = openai.AzureOpenAI(
                 azure_endpoint=azure_endpoint,
                 azure_deployment=azure_deployment,
                 api_key=azure_api_key,
@@ -153,7 +167,7 @@ class Maitai:
                 _strict_response_validation=_strict_response_validation,
             )
         elif openai_api_key or os.environ.get("OPENAI_API_KEY"):
-            self._client = openai.OpenAI(
+            self._openai_client = openai.OpenAI(
                 api_key=openai_api_key or os.environ.get("OPENAI_API_KEY"),
                 organization=organization,
                 project=project,
@@ -165,6 +179,11 @@ class Maitai:
                 http_client=http_client,
                 _strict_response_validation=_strict_response_validation,
             )
+        if groq_api_key or config.auth_keys.groq_api_key:
+            self._groq_client = groq.Groq(
+                api_key=groq_api_key or config.auth_keys.groq_api_key.key_value,
+            )
+
         client_params = None
         if base_url or default_headers or default_query:
             client_params = ClientParams(
@@ -173,21 +192,25 @@ class Maitai:
                 default_query=default_query or {},
                 azure_params=azure_params,
             )
-        self.chat = Chat(self._client, client_params)
+        self.chat = Chat(self._openai_client, self._groq_client, client_params)
 
 
 class Chat:
-    def __init__(self, client=None, client_params: ClientParams = None):
-        self.completions = Completions(client, client_params)
+    def __init__(
+        self, openai_client=None, groq_client=None, client_params: ClientParams = None
+    ):
+        self.completions = Completions(openai_client, groq_client, client_params)
 
 
 class Completions:
     def __init__(
         self,
-        client: Optional[openai.Client] = None,
+        openai_client: Optional[openai.Client] = None,
+        groq_client: Optional[groq.Groq] = None,
         client_params: Optional[ClientParams] = None,
     ):
-        self._client = client
+        self._openai_client = openai_client
+        self._groq_client = groq_client
         self._client_params = client_params or ClientParams()
         self.evaluator = Evaluator()
         self.inference = Inference()
@@ -456,17 +479,70 @@ class Completions:
                 if safe_mode:
                     raise e
                 traceback.print_exc()
-                print("Maitai issue, retrying on client side")
+                logger.warning("Maitai issue, retrying on client side")
                 server_side_inference = False
         if not server_side_inference:
+            local_params = {
+                "messages": messages,
+                "model": model,
+                "frequency_penalty": frequency_penalty,
+                "function_call": function_call,
+                "functions": functions,
+                "logit_bias": logit_bias,
+                "logprobs": logprobs,
+                "max_tokens": max_tokens,
+                "n": n,
+                "presence_penalty": presence_penalty,
+                "reasoning_effort": reasoning_effort,
+                "response_format": response_format,
+                "seed": seed,
+                "stop": stop,
+                "stream": stream,
+                "stream_options": stream_options,
+                "temperature": temperature,
+                "tool_choice": tool_choice,
+                "tools": tools,
+                "top_logprobs": top_logprobs,
+                "top_p": top_p,
+                "user": user,
+                "extra_headers": extra_headers,
+                "extra_query": extra_query,
+                "extra_body": extra_body,
+                "timeout": timeout,
+                "parallel_tool_calls": parallel_tool_calls,
+            }
+            groq_model = False
+            if "groq:" in model or "maitai:" in model or "llama" in model:
+                groq_model = True
+                model = model.replace("groq:", "").replace("maitai:", "")
             if "openai:" in model:
                 model = model.replace("openai:", "")
-            if self._client is None:
-                self._client = openai.OpenAI(
-                    default_query=self._client_params.default_query or None,
-                    default_headers=self._client_params.default_headers or None,
-                    base_url=self._client_params.base_url or None,
-                )
+
+            if groq_model:
+                if self._groq_client is None:
+                    if (
+                        not config.auth_keys.groq_api_key
+                        or not config.auth_keys.groq_api_key.key_value
+                    ):
+                        raise ValueError(
+                            "Groq API key is required for Groq models but was not provided"
+                        )
+
+                    self._groq_client = groq.Groq(
+                        api_key=config.auth_keys.groq_api_key.key_value,
+                    )
+                client_to_use = self._groq_client
+                local_params = convert_params_to_groq(local_params)
+            else:
+                if self._openai_client is None:
+                    self._openai_client = openai.OpenAI(
+                        api_key=config.auth_keys.openai_api_key.key_value,
+                        default_headers=self._client_params.default_headers or None,
+                        default_query=self._client_params.default_query or None,
+                        base_url=self._client_params.base_url or None,
+                    )
+                client_to_use = self._openai_client
+
             if hasattr(response_format, "model_json_schema"):
                 response_format = {
                     "type": "json_schema",
@@ -475,35 +551,7 @@ class Completions:
             if isinstance(tools, Tools):
                 tools = tools.get_tool_definitions()
 
-            response = self._client.chat.completions.create(
-                messages=messages,
-                model=model,
-                frequency_penalty=frequency_penalty,
-                function_call=function_call,
-                functions=functions,
-                logit_bias=logit_bias,
-                logprobs=logprobs,
-                max_tokens=max_tokens,
-                n=n,
-                presence_penalty=presence_penalty,
-                reasoning_effort=reasoning_effort,
-                response_format=response_format,
-                seed=seed,
-                stop=stop,
-                stream=stream,
-                stream_options=stream_options,
-                temperature=temperature,
-                tool_choice=tool_choice,
-                tools=tools,
-                top_logprobs=top_logprobs,
-                top_p=top_p,
-                user=user,
-                extra_headers=extra_headers,
-                extra_query=extra_query,
-                extra_body=extra_body,
-                timeout=timeout,
-                parallel_tool_calls=parallel_tool_calls,
-            )
+            response = client_to_use.chat.completions.create(**local_params)
             if stream:
                 return self._process_openai_stream(
                     session_id,
@@ -656,7 +704,7 @@ class Completions:
             ):
                 retry_args["model"] = retry_args["fallback_model"]
             if "gpt" in retry_args.get("model", ""):
-                print("Maitai issue, retrying on client side")
+                logger.warning("Maitai issue, retrying on client side")
                 retry_args["model"] = retry_args["model"].replace("openai:", "")
                 response = self.create(**retry_args)
                 for resp in response:
